#include <YoggieEngine.h>
#include "OpenglAPI.h"
namespace YoggieEngine {
	
	GLenum glCheckError_(const char* file, int line) {
		GLenum errorCode;
		while ((errorCode = glGetError()) != GL_NO_ERROR) {
			std::string error;
			switch (errorCode)
			{
			case GL_INVALID_ENUM: error = "INVALID_ENUM"; break;
			case GL_INVALID_VALUE: error = "INVALID_VALUE"; break;
			case GL_INVALID_OPERATION: error = "INVALID_OPERATION"; break;
			case GL_STACK_OVERFLOW: error = "STACK_OVERFLOW"; break;
			case GL_STACK_UNDERFLOW: error = "STACK_UNDERFLOW"; break;
			case GL_OUT_OF_MEMORY: error = "OUT_OF_MEMORY"; break;
			case GL_INVALID_FRAMEBUFFER_OPERATION: error = "INVALID_FRAMEBUFFER_OPERATION"; break;
			};
			spdlog::error("{0} | {1} ({2})", error, file, line);
		}
		return errorCode;
	}

#ifdef DEBUG
#define glCheckError() glCheckError_(__FILE__, __LINE__)
#else 
#define glCheckError()
#endif

	void OpenGLApi::DrawTriangles(Render3DComponent rc) {

		glBindVertexArray(rc.VAO);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, rc.IBO);
		glDrawElements(GL_TRIANGLES, static_cast<unsigned int> (rc.mesh.elements.size()), GL_UNSIGNED_INT, 0);
		glCheckError();
		glBindVertexArray(0);
	}



	void OpenGLApi::DrawCubeMap(unsigned int VertexAttributeObject, CubeMap CubeTexture) {

		glDepthMask(GL_FALSE);
		glBindTexture(GL_TEXTURE_CUBE_MAP, CubeTexture.getID());
		//glCheckError(); // INVALID ENUM FOR SOME REASON
		glBindVertexArray(VertexAttributeObject);
		glDrawArrays(GL_TRIANGLES, 0, 36);
		//glCheckError();
		glBindVertexArray(0);
		glDepthMask(GL_TRUE);

	}


}
